@@ -69,21 +69,21 @@ static const char* PyUpb_UnknownFieldSet_BuildMessageSetItem(
   PyObject* msg = NULL;
   while (!upb_EpsCopyInputStream_IsDone(stream, &ptr)) {
     uint32_t tag;
-    ptr = upb_WireReader_ReadTag(ptr, &tag);
+    ptr = upb_WireReader_ReadTag(ptr, &tag, stream);
     if (!ptr) goto err;
     switch (tag) {
       case kUpb_MessageSet_EndItemTag:
         goto done;
       case kUpb_MessageSet_TypeIdTag: {
         uint64_t tmp;
-        ptr = upb_WireReader_ReadVarint(ptr, &tmp);
+        ptr = upb_WireReader_ReadVarint(ptr, &tmp, stream);
         if (!ptr) goto err;
         if (!type_id) type_id = tmp;
         break;
       }
       case kUpb_MessageSet_MessageTag: {
         int size;
-        ptr = upb_WireReader_ReadSize(ptr, &size);
+        ptr = upb_WireReader_ReadSize(ptr, &size, stream);
         if (!upb_EpsCopyInputStream_CheckDataSizeAvailable(stream, ptr, size)) {
           goto err;
         }
@@ -124,7 +124,7 @@ static const char* PyUpb_UnknownFieldSet_BuildMessageSet(
     const char* ptr) {
   while (!upb_EpsCopyInputStream_IsDone(stream, &ptr)) {
     uint32_t tag;
-    ptr = upb_WireReader_ReadTag(ptr, &tag);
+    ptr = upb_WireReader_ReadTag(ptr, &tag, stream);
     if (!ptr) goto err;
     if (tag == kUpb_MessageSet_StartItemTag) {
       ptr = PyUpb_UnknownFieldSet_BuildMessageSetItem(self, stream, ptr);
@@ -153,26 +153,26 @@ static const char* PyUpb_UnknownFieldSet_BuildValue(
   switch (wire_type) {
     case kUpb_WireType_Varint: {
       uint64_t val;
-      ptr = upb_WireReader_ReadVarint(ptr, &val);
+      ptr = upb_WireReader_ReadVarint(ptr, &val, stream);
       if (!ptr) return NULL;
       *data = PyLong_FromUnsignedLongLong(val);
       return ptr;
     }
     case kUpb_WireType_64Bit: {
       uint64_t val;
-      ptr = upb_WireReader_ReadFixed64(ptr, &val);
+      ptr = upb_WireReader_ReadFixed64(ptr, &val, stream);
       *data = PyLong_FromUnsignedLongLong(val);
       return ptr;
     }
     case kUpb_WireType_32Bit: {
       uint32_t val;
-      ptr = upb_WireReader_ReadFixed32(ptr, &val);
+      ptr = upb_WireReader_ReadFixed32(ptr, &val, stream);
       *data = PyLong_FromUnsignedLongLong(val);
       return ptr;
     }
     case kUpb_WireType_Delimited: {
       int size;
-      ptr = upb_WireReader_ReadSize(ptr, &size);
+      ptr = upb_WireReader_ReadSize(ptr, &size, stream);
       if (!upb_EpsCopyInputStream_CheckDataSizeAvailable(stream, ptr, size)) {
         return NULL;
       }
@@ -203,7 +203,7 @@ static const char* PyUpb_UnknownFieldSet_Build(PyUpb_UnknownFieldSet* self,
   PyUpb_ModuleState* s = PyUpb_ModuleState_Get();
   while (!upb_EpsCopyInputStream_IsDone(stream, &ptr)) {
     uint32_t tag;
-    ptr = upb_WireReader_ReadTag(ptr, &tag);
+    ptr = upb_WireReader_ReadTag(ptr, &tag, stream);
     if (!ptr) goto err;
     PyObject* data = NULL;
     int field_number = upb_WireReader_GetFieldNumber(tag);
