@@ -93,9 +93,15 @@ const upb_DefPool* DescriptorPool_GetSymtab(VALUE desc_pool_rb) {
   return pool->symtab;
 }
 
+/**
+ * ruby-doc: DescriptorPool
+ *
+ * A DescriptorPool is the registry of all known Protobuf descriptor objects.
+ *
+ */
+
 /*
- * call-seq:
- *     DescriptorPool.new => pool
+ * ruby-doc: DescriptorPool.new
  *
  * Creates a new, empty, descriptor pool.
  */
@@ -112,10 +118,14 @@ static VALUE DescriptorPool_alloc(VALUE klass) {
 }
 
 /*
- * call-seq:
- *     DescriptorPool.add_serialized_file(serialized_file_proto)
+ * ruby-doc: DescriptorPool#add_serialized_file
  *
- * Adds the given serialized FileDescriptorProto to the pool.
+ * Adds the given serialized
+ * {https://protobuf.com/docs/descriptors#file-descriptors FileDescriptorProto}
+ * to the pool.
+ *
+ * @param serialized_file_proto [String]
+ * @return [FileDescriptor]
  */
 VALUE DescriptorPool_add_serialized_file(VALUE _self,
                                          VALUE serialized_file_proto) {
@@ -143,11 +153,14 @@ VALUE DescriptorPool_add_serialized_file(VALUE _self,
 }
 
 /*
- * call-seq:
- *     DescriptorPool.lookup(name) => descriptor
+ * ruby-doc: DescriptorPool#lookup
  *
- * Finds a Descriptor, EnumDescriptor or FieldDescriptor by name and returns it,
- * or nil if none exists with the given name.
+ * Finds a {Descriptor}, {EnumDescriptor},
+ * {FieldDescriptor} or {ServiceDescriptor} by
+ * name and returns it, or nil if none exists with the given name.
+ *
+ * @param name [String]
+ * @return [Descriptor,EnumDescriptor,FieldDescriptor,ServiceDescriptor]
  */
 static VALUE DescriptorPool_lookup(VALUE _self, VALUE name) {
   DescriptorPool* self = ruby_to_DescriptorPool(_self);
@@ -156,6 +169,7 @@ static VALUE DescriptorPool_lookup(VALUE _self, VALUE name) {
   const upb_EnumDef* enumdef;
   const upb_FieldDef* fielddef;
   const upb_ServiceDef* servicedef;
+  const upb_FileDef* filedef;
 
   msgdef = upb_DefPool_FindMessageByName(self->symtab, name_str);
   if (msgdef) {
@@ -177,17 +191,23 @@ static VALUE DescriptorPool_lookup(VALUE _self, VALUE name) {
     return get_servicedef_obj(_self, servicedef);
   }
 
+  filedef = upb_DefPool_FindFileByName(self->symtab, name_str);
+  if (filedef) {
+    return get_filedef_obj(_self, filedef);
+  }
+
   return Qnil;
 }
 
 /*
- * call-seq:
- *     DescriptorPool.generated_pool => descriptor_pool
+ * ruby-doc: DescriptorPool.generated_pool
  *
- * Class method that returns the global DescriptorPool. This is a singleton into
- * which generated-code message and enum types are registered. The user may also
- * register types in this pool for convenience so that they do not have to hold
- * a reference to a private pool instance.
+ * Class method that returns the global {DescriptorPool}. This is a singleton
+ * into which generated-code message and enum types are registered. The user may
+ * also register types in this pool for convenience so that they do not have to
+ * hold a reference to a private pool instance.
+ *
+ * @return [DescriptorPool]
  */
 static VALUE DescriptorPool_generated_pool(VALUE _self) {
   return generated_pool;
@@ -284,8 +304,13 @@ static VALUE decode_options(VALUE self, const char* option_type, int size,
 }
 
 /*
- * call-seq:
- *     Descriptor.new => descriptor
+ * ruby-doc: Descriptor
+ *
+ * A Descriptor provides information about a given Protobuf definition.
+ */
+
+/*
+ * ruby-doc: Descriptor.initialize
  *
  * Creates a new, empty, message type descriptor. At a minimum, its name must be
  * set before it is added to a pool. It cannot be used to create messages until
@@ -323,10 +348,11 @@ static VALUE Descriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *    Descriptor.file_descriptor
+ * ruby-doc: Descriptor#file_descriptor
  *
- * Returns the FileDescriptor object this message belongs to.
+ * Returns the {FileDescriptor} object this message belongs to.
+ *
+ * @return [FileDescriptor]
  */
 static VALUE Descriptor_file_descriptor(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -335,11 +361,12 @@ static VALUE Descriptor_file_descriptor(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     Descriptor.name => name
+ * ruby-doc: Descriptor#name
  *
  * Returns the name of this message type as a fully-qualified string (e.g.,
  * My.Package.MessageType).
+ *
+ * @return [String]
  */
 static VALUE Descriptor_name(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -347,10 +374,12 @@ static VALUE Descriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     Descriptor.each(&block)
+ * ruby-doc: Descriptor#each
  *
  * Iterates over fields in this message type, yielding to the block on each one.
+ *
+ * @yield [FieldDescriptor]
+ * @return [nil]
  */
 static VALUE Descriptor_each(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -365,11 +394,13 @@ static VALUE Descriptor_each(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     Descriptor.lookup(name) => FieldDescriptor
+ * ruby-doc: Descriptor#lookup
  *
  * Returns the field descriptor for the field with the given name, if present,
  * or nil if none.
+ *
+ * @param name [String]
+ * @return [FieldDescriptor]
  */
 static VALUE Descriptor_lookup(VALUE _self, VALUE name) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -382,11 +413,13 @@ static VALUE Descriptor_lookup(VALUE _self, VALUE name) {
 }
 
 /*
- * call-seq:
- *     Descriptor.each_oneof(&block) => nil
+ * ruby-doc: Descriptor#each_oneof
  *
  * Invokes the given block for each oneof in this message type, passing the
- * corresponding OneofDescriptor.
+ * corresponding {OneofDescriptor}.
+ *
+ * @yield [OneofDescriptor]
+ * @return [nil]
  */
 static VALUE Descriptor_each_oneof(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -401,11 +434,13 @@ static VALUE Descriptor_each_oneof(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     Descriptor.lookup_oneof(name) => OneofDescriptor
+ * ruby-doc: Descriptor#lookup_oneof
  *
  * Returns the oneof descriptor for the oneof with the given name, if present,
  * or nil if none.
+ *
+ * @param name [String]
+ * @return [OneofDescriptor]
  */
 static VALUE Descriptor_lookup_oneof(VALUE _self, VALUE name) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -418,10 +453,11 @@ static VALUE Descriptor_lookup_oneof(VALUE _self, VALUE name) {
 }
 
 /*
- * call-seq:
- *     Descriptor.msgclass => message_klass
+ * ruby-doc: Descriptor#msgclass
  *
  * Returns the Ruby class created for this message type.
+ *
+ * @return [Class<Google::Protobuf::AbstractMessage>]
  */
 static VALUE Descriptor_msgclass(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -432,10 +468,13 @@ static VALUE Descriptor_msgclass(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     Descriptor.options => options
+ * ruby-doc: Descriptor#options
  *
- * Returns the `MessageOptions` for this `Descriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L571
+ * MessageOptions} for this {Descriptor}.
+ *
+ * @return [MessageOptions]
  */
 static VALUE Descriptor_options(VALUE _self) {
   Descriptor* self = ruby_to_Descriptor(_self);
@@ -451,6 +490,30 @@ static VALUE Descriptor_options(VALUE _self) {
   return message_options;
 }
 
+/*
+ * ruby-doc: Descriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L147
+ * DescriptorProto} of this {Descriptor}.
+ *
+ * @return [DescriptorProto]
+ */
+static VALUE Descriptor_to_proto(VALUE _self) {
+  Descriptor* self = ruby_to_Descriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_DescriptorProto* proto =
+      upb_MessageDef_ToProto(self->msgdef, arena);
+  size_t size;
+  const char* serialized =
+      google_protobuf_DescriptorProto_serialize(proto, arena, &size);
+  VALUE proto_class = rb_path2class("Google::Protobuf::DescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void Descriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "Descriptor", rb_cObject);
   rb_define_alloc_func(klass, Descriptor_alloc);
@@ -463,6 +526,7 @@ static void Descriptor_register(VALUE module) {
   rb_define_method(klass, "name", Descriptor_name, 0);
   rb_define_method(klass, "file_descriptor", Descriptor_file_descriptor, 0);
   rb_define_method(klass, "options", Descriptor_options, 0);
+  rb_define_method(klass, "to_proto", Descriptor_to_proto, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cDescriptor);
   cDescriptor = klass;
@@ -506,9 +570,15 @@ static VALUE FileDescriptor_alloc(VALUE klass) {
   return ret;
 }
 
+/**
+ * ruby-doc: FileDescriptor
+ *
+ * A FileDescriptor provides information about all Protobuf definitions in a
+ * particular file.
+ */
+
 /*
- * call-seq:
- *     FileDescriptor.new => file
+ * ruby-doc: FileDescriptor#initialize
  *
  * Returns a new file descriptor. May
  * to a builder.
@@ -529,10 +599,11 @@ static VALUE FileDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *     FileDescriptor.name => name
+ * ruby-doc: FileDescriptor#name
  *
  * Returns the name of the file.
+ *
+ * @return [String]
  */
 static VALUE FileDescriptor_name(VALUE _self) {
   FileDescriptor* self = ruby_to_FileDescriptor(_self);
@@ -541,10 +612,13 @@ static VALUE FileDescriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FileDescriptor.options => options
+ * ruby-doc: FileDescriptor#options
  *
- * Returns the `FileOptions` for this `FileDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L442
+ * FileOptions} for this {FileDescriptor}.
+ *
+ * @return [FileOptions]
  */
 static VALUE FileDescriptor_options(VALUE _self) {
   FileDescriptor* self = ruby_to_FileDescriptor(_self);
@@ -558,12 +632,40 @@ static VALUE FileDescriptor_options(VALUE _self) {
   return file_options;
 }
 
+/*
+ * ruby-doc: FileDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L104
+ * FileDescriptorProto} of this {FileDescriptor}.
+ *
+ * @return [FileDescriptorProto]
+ */
+static VALUE FileDescriptor_to_proto(VALUE _self) {
+  FileDescriptor* self = ruby_to_FileDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_FileDescriptorProto* file_proto =
+      upb_FileDef_ToProto(self->filedef, arena);
+
+  size_t size;
+  const char* serialized =
+      google_protobuf_FileDescriptorProto_serialize(file_proto, arena, &size);
+
+  VALUE file_proto_class =
+      rb_path2class("Google::Protobuf::FileDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, file_proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void FileDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "FileDescriptor", rb_cObject);
   rb_define_alloc_func(klass, FileDescriptor_alloc);
   rb_define_method(klass, "initialize", FileDescriptor_initialize, 3);
   rb_define_method(klass, "name", FileDescriptor_name, 0);
   rb_define_method(klass, "options", FileDescriptor_options, 0);
+  rb_define_method(klass, "to_proto", FileDescriptor_to_proto, 0);
   rb_gc_register_address(&cFileDescriptor);
   cFileDescriptor = klass;
 }
@@ -598,9 +700,15 @@ static FieldDescriptor* ruby_to_FieldDescriptor(VALUE val) {
   return ret;
 }
 
+/**
+ * ruby-doc: FieldDescriptor
+ *
+ * A FieldDescriptor provides information about the Protobuf definition of a
+ * field inside a {Descriptor}.
+ */
+
 /*
- * call-seq:
- *     FieldDescriptor.new => field
+ * ruby-doc: FieldDescriptor#initialize
  *
  * Returns a new field descriptor. Its name, type, etc. must be set before it is
  * added to a message type.
@@ -634,10 +742,11 @@ static VALUE FieldDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.name => name
+ * ruby-doc: FieldDescriptor#name
  *
  * Returns the name of this field.
+ *
+ * @return [String]
  */
 static VALUE FieldDescriptor_name(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -702,14 +811,15 @@ static VALUE descriptortype_to_ruby(upb_FieldType type) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.type => type
+ * ruby-doc: FieldDescriptor#type
  *
  * Returns this field's type, as a Ruby symbol, or nil if not yet set.
  *
  * Valid field types are:
  *     :int32, :int64, :uint32, :uint64, :float, :double, :bool, :string,
  *     :bytes, :message.
+ *
+ * @return [Symbol]
  */
 static VALUE FieldDescriptor__type(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -717,10 +827,11 @@ static VALUE FieldDescriptor__type(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.default => default
+ * ruby-doc: FieldDescriptor#default
  *
  * Returns this field's default, as a Ruby object, or nil if not yet set.
+ *
+ * @return [Object,nil]
  */
 static VALUE FieldDescriptor_default(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -735,10 +846,11 @@ static VALUE FieldDescriptor_default(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.has_presence? => bool
+ * ruby-doc: FieldDescriptor.has_presence?
  *
  * Returns whether this field tracks presence.
+ *
+ * @return [Boolean]
  */
 static VALUE FieldDescriptor_has_presence(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -746,10 +858,33 @@ static VALUE FieldDescriptor_has_presence(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.is_packed? => bool
+ * ruby-doc: FieldDescriptor#required?
+ *
+ * Returns whether this is a required field.
+ *
+ * @return [Boolean]
+ */
+static VALUE FieldDescriptor_is_required(VALUE _self) {
+  FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
+  return upb_FieldDef_IsRequired(self->fielddef) ? Qtrue : Qfalse;
+}
+
+/*
+ * ruby-doc: FieldDescriptor#repeated?
+ *
+ * Returns whether this is a repeated field.
+ * @return [Boolean]
+ */
+static VALUE FieldDescriptor_is_repeated(VALUE _self) {
+  FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
+  return upb_FieldDef_IsRepeated(self->fielddef) ? Qtrue : Qfalse;
+}
+
+/*
+ * ruby-doc: FieldDescriptor#is_packed?
  *
  * Returns whether this is a repeated field that uses packed encoding.
+ * @return [Boolean]
  */
 static VALUE FieldDescriptor_is_packed(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -757,10 +892,11 @@ static VALUE FieldDescriptor_is_packed(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.json_name => json_name
+ * ruby-doc: FieldDescriptor#json_name
  *
  * Returns this field's json_name, as a Ruby string, or nil if not yet set.
+ *
+ * @return [String,nil]
  */
 static VALUE FieldDescriptor_json_name(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -770,13 +906,14 @@ static VALUE FieldDescriptor_json_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.label => label
+ * ruby-doc: FieldDescriptor#label
  *
  * Returns this field's label (i.e., plurality), as a Ruby symbol.
- *
  * Valid field labels are:
- *     :optional, :repeated
+ *   :optional, :repeated
+ *
+ * @return [Symbol]
+ * @deprecated Use {#repeated?} or {#required?} instead.
  */
 static VALUE FieldDescriptor_label(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -796,10 +933,11 @@ static VALUE FieldDescriptor_label(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.number => number
+ * ruby-doc: FieldDescriptor#number
  *
  * Returns the tag number for this field.
+ *
+ * @return [Integer]
  */
 static VALUE FieldDescriptor_number(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -807,13 +945,14 @@ static VALUE FieldDescriptor_number(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.submsg_name => submsg_name
+ * ruby-doc: FieldDescriptor#submsg_name
  *
  * Returns the name of the message or enum type corresponding to this field, if
  * it is a message or enum field (respectively), or nil otherwise. This type
  * name will be resolved within the context of the pool to which the containing
  * message type is added.
+ *
+ * @return [String,nil]
  */
 static VALUE FieldDescriptor_submsg_name(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -830,13 +969,14 @@ static VALUE FieldDescriptor_submsg_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.subtype => message_or_enum_descriptor
+ * ruby-doc: FieldDescriptor#subtype
  *
  * Returns the message or enum descriptor corresponding to this field's type if
  * it is a message or enum field, respectively, or nil otherwise. Cannot be
  * called *until* the containing message type is added to a pool (and thus
  * resolved).
+ *
+ * @return [Descriptor,EnumDescriptor,nil]
  */
 static VALUE FieldDescriptor_subtype(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -853,11 +993,13 @@ static VALUE FieldDescriptor_subtype(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.get(message) => value
+ * ruby-doc: FieldDescriptor#get
  *
  * Returns the value set for this field on the given message. Raises an
  * exception if message is of the wrong type.
+ *
+ * @param message [AbstractMessage]
+ * @return [Object]
  */
 static VALUE FieldDescriptor_get(VALUE _self, VALUE msg_rb) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -873,11 +1015,13 @@ static VALUE FieldDescriptor_get(VALUE _self, VALUE msg_rb) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.has?(message) => boolean
+ * ruby-doc: FieldDescriptor.has?
  *
  * Returns whether the value is set on the given message. Raises an
  * exception when calling for fields that do not have presence.
+ *
+ * @param message [AbstractMessage]
+ * @return [Boolean]
  */
 static VALUE FieldDescriptor_has(VALUE _self, VALUE msg_rb) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -894,10 +1038,12 @@ static VALUE FieldDescriptor_has(VALUE _self, VALUE msg_rb) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.clear(message)
+ * ruby-doc: FieldDescriptor#clear
  *
  * Clears the field from the message if it's set.
+ *
+ * @param message [AbstractMessage]
+ * @return [nil]
  */
 static VALUE FieldDescriptor_clear(VALUE _self, VALUE msg_rb) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -913,12 +1059,14 @@ static VALUE FieldDescriptor_clear(VALUE _self, VALUE msg_rb) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.set(message, value)
+ * ruby-doc: FieldDescriptor#set
  *
  * Sets the value corresponding to this field to the given value on the given
  * message. Raises an exception if message is of the wrong type. Performs the
  * ordinary type-checks for field setting.
+ *
+ * @param message [AbstractMessage]
+ * @param value [Object]
  */
 static VALUE FieldDescriptor_set(VALUE _self, VALUE msg_rb, VALUE value) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -938,10 +1086,13 @@ static VALUE FieldDescriptor_set(VALUE _self, VALUE msg_rb, VALUE value) {
 }
 
 /*
- * call-seq:
- *     FieldDescriptor.options => options
+ * ruby-doc: FieldDescriptor#options
  *
- * Returns the `FieldOptions` for this `FieldDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L656
+ * FieldOptions} for this {FieldDescriptor}.
+ *
+ * @return [FieldOptions]
  */
 static VALUE FieldDescriptor_options(VALUE _self) {
   FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
@@ -956,6 +1107,30 @@ static VALUE FieldDescriptor_options(VALUE _self) {
   return field_options;
 }
 
+/*
+ * ruby-doc: FieldDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L236
+ * FieldDescriptorProto} of this {FieldDescriptor}.
+ *
+ * @return [FieldDescriptorProto]
+ */
+static VALUE FieldDescriptor_to_proto(VALUE _self) {
+  FieldDescriptor* self = ruby_to_FieldDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_FieldDescriptorProto* proto =
+      upb_FieldDef_ToProto(self->fielddef, arena);
+  size_t size;
+  const char* serialized =
+      google_protobuf_FieldDescriptorProto_serialize(proto, arena, &size);
+  VALUE proto_class = rb_path2class("Google::Protobuf::FieldDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void FieldDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "FieldDescriptor", rb_cObject);
   rb_define_alloc_func(klass, FieldDescriptor_alloc);
@@ -964,6 +1139,8 @@ static void FieldDescriptor_register(VALUE module) {
   rb_define_method(klass, "type", FieldDescriptor__type, 0);
   rb_define_method(klass, "default", FieldDescriptor_default, 0);
   rb_define_method(klass, "has_presence?", FieldDescriptor_has_presence, 0);
+  rb_define_method(klass, "required?", FieldDescriptor_is_required, 0);
+  rb_define_method(klass, "repeated?", FieldDescriptor_is_repeated, 0);
   rb_define_method(klass, "is_packed?", FieldDescriptor_is_packed, 0);
   rb_define_method(klass, "json_name", FieldDescriptor_json_name, 0);
   rb_define_method(klass, "label", FieldDescriptor_label, 0);
@@ -975,6 +1152,7 @@ static void FieldDescriptor_register(VALUE module) {
   rb_define_method(klass, "get", FieldDescriptor_get, 1);
   rb_define_method(klass, "set", FieldDescriptor_set, 2);
   rb_define_method(klass, "options", FieldDescriptor_options, 0);
+  rb_define_method(klass, "to_proto", FieldDescriptor_to_proto, 0);
   rb_gc_register_address(&cFieldDescriptor);
   cFieldDescriptor = klass;
 }
@@ -1009,9 +1187,15 @@ static OneofDescriptor* ruby_to_OneofDescriptor(VALUE val) {
   return ret;
 }
 
+/**
+ * ruby-doc: OneofDescriptor
+ *
+ * A OneofDescriptor provides information about the Protobuf definition of a
+ * oneof inside a {Descriptor}.
+ */
+
 /*
- * call-seq:
- *     OneofDescriptor.new => oneof_descriptor
+ * ruby-doc: OneofDescriptor#initialize
  *
  * Creates a new, empty, oneof descriptor. The oneof may only be modified prior
  * to being added to a message descriptor which is subsequently added to a pool.
@@ -1046,10 +1230,11 @@ static VALUE OneofDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *     OneofDescriptor.name => name
+ * ruby-doc: OneofDescriptor#name
  *
  * Returns the name of this oneof.
+ *
+ * @return [String]
  */
 static VALUE OneofDescriptor_name(VALUE _self) {
   OneofDescriptor* self = ruby_to_OneofDescriptor(_self);
@@ -1057,10 +1242,12 @@ static VALUE OneofDescriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     OneofDescriptor.each(&block) => nil
+ * ruby-doc: OneofDescriptor#each
  *
  * Iterates through fields in this oneof, yielding to the block on each one.
+ *
+ * @yield [FieldDescriptor]
+ * @return [nil]
  */
 static VALUE OneofDescriptor_each(VALUE _self) {
   OneofDescriptor* self = ruby_to_OneofDescriptor(_self);
@@ -1075,10 +1262,13 @@ static VALUE OneofDescriptor_each(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     OneofDescriptor.options => options
+ * ruby-doc: OneofDescriptor#options
  *
- * Returns the `OneofOptions` for this `OneofDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L824
+ * OneofOptions} for this {OneofDescriptor}.
+ *
+ * @return [OneofOptions]
  */
 static VALUE OneOfDescriptor_options(VALUE _self) {
   OneofDescriptor* self = ruby_to_OneofDescriptor(_self);
@@ -1093,6 +1283,30 @@ static VALUE OneOfDescriptor_options(VALUE _self) {
   return oneof_options;
 }
 
+/*
+ * ruby-doc: OneofDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L343
+ * OneofDescriptorProto} of this {OneofDescriptor}.
+ *
+ * @return [OneofDescriptorProto]
+ */
+static VALUE OneOfDescriptor_to_proto(VALUE _self) {
+  OneofDescriptor* self = ruby_to_OneofDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_OneofDescriptorProto* proto =
+      upb_OneofDef_ToProto(self->oneofdef, arena);
+  size_t size;
+  const char* serialized =
+      google_protobuf_OneofDescriptorProto_serialize(proto, arena, &size);
+  VALUE proto_class = rb_path2class("Google::Protobuf::OneofDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void OneofDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "OneofDescriptor", rb_cObject);
   rb_define_alloc_func(klass, OneofDescriptor_alloc);
@@ -1100,6 +1314,7 @@ static void OneofDescriptor_register(VALUE module) {
   rb_define_method(klass, "name", OneofDescriptor_name, 0);
   rb_define_method(klass, "each", OneofDescriptor_each, 0);
   rb_define_method(klass, "options", OneOfDescriptor_options, 0);
+  rb_define_method(klass, "to_proto", OneOfDescriptor_to_proto, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cOneofDescriptor);
   cOneofDescriptor = klass;
@@ -1152,6 +1367,13 @@ const upb_EnumDef* EnumDescriptor_GetEnumDef(VALUE enum_desc_rb) {
   return desc->enumdef;
 }
 
+/**
+ * ruby-doc: EnumDescriptor
+ *
+ * An EnumDescriptor provides information about the Protobuf definition of an
+ * enum inside a {Descriptor}.
+ */
+
 /*
  * call-seq:
  *    EnumDescriptor.new(c_only_cookie, ptr) => EnumDescriptor
@@ -1174,10 +1396,11 @@ static VALUE EnumDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *    EnumDescriptor.file_descriptor
+ * ruby-doc: EnumDescriptor#file_descriptor
  *
- * Returns the FileDescriptor object this enum belongs to.
+ * Returns the {FileDescriptor} object this enum belongs to.
+ *
+ * @return [FileDescriptor]
  */
 static VALUE EnumDescriptor_file_descriptor(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1186,10 +1409,11 @@ static VALUE EnumDescriptor_file_descriptor(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.is_closed? => bool
+ * ruby-doc: EnumDescriptor#is_closed?
  *
  * Returns whether this enum is open or closed.
+ *
+ * @return [Boolean]
  */
 static VALUE EnumDescriptor_is_closed(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1197,10 +1421,11 @@ static VALUE EnumDescriptor_is_closed(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.name => name
+ * ruby-doc: EnumDescriptor#name
  *
  * Returns the name of this enum type.
+ *
+ * @return [String]
  */
 static VALUE EnumDescriptor_name(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1208,11 +1433,13 @@ static VALUE EnumDescriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.lookup_name(name) => value
+ * ruby-doc: EnumDescriptor#lookup_name
  *
  * Returns the numeric value corresponding to the given key name (as a Ruby
  * symbol), or nil if none.
+ *
+ * @param name [Symbol]
+ * @return [Integer,nil]
  */
 static VALUE EnumDescriptor_lookup_name(VALUE _self, VALUE name) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1227,11 +1454,13 @@ static VALUE EnumDescriptor_lookup_name(VALUE _self, VALUE name) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.lookup_value(name) => value
+ * ruby-doc: EnumDescriptor#lookup_value
  *
  * Returns the key name (as a Ruby symbol) corresponding to the integer value,
  * or nil if none.
+ *
+ * @param name [Integer]
+ * @return [Symbol,nil]
  */
 static VALUE EnumDescriptor_lookup_value(VALUE _self, VALUE number) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1246,11 +1475,13 @@ static VALUE EnumDescriptor_lookup_value(VALUE _self, VALUE number) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.each(&block)
+ * ruby-doc: EnumDescriptor#each
  *
  * Iterates over key => value mappings in this enum's definition, yielding to
  * the block with (key, value) arguments for each one.
+ *
+ * @yield [Symbol, Integer]
+ * @return [nil]
  */
 static VALUE EnumDescriptor_each(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1267,10 +1498,11 @@ static VALUE EnumDescriptor_each(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.enummodule => module
+ * ruby-doc: EnumDescriptor#enummodule
  *
  * Returns the Ruby module corresponding to this enum type.
+ *
+ * @return [Module]
  */
 static VALUE EnumDescriptor_enummodule(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1281,10 +1513,13 @@ static VALUE EnumDescriptor_enummodule(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     EnumDescriptor.options => options
+ * ruby-doc: EnumDescriptor#options
  *
- * Returns the `EnumOptions` for this `EnumDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L838
+ * EnumOptions} for this {EnumDescriptor}.
+ *
+ * @return [EnumOptions]
  */
 static VALUE EnumDescriptor_options(VALUE _self) {
   EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
@@ -1298,6 +1533,31 @@ static VALUE EnumDescriptor_options(VALUE _self) {
   return enum_options;
 }
 
+/*
+ * ruby-doc: EnumDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L349
+ * EnumDescriptorProto} of this {EnumDescriptor}.
+ * @return [EnumDescriptorProto]
+ */
+static VALUE EnumDescriptor_to_proto(VALUE _self) {
+  EnumDescriptor* self = ruby_to_EnumDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_EnumDescriptorProto* proto =
+      upb_EnumDef_ToProto(self->enumdef, arena);
+
+  size_t size;
+  const char* serialized =
+      google_protobuf_EnumDescriptorProto_serialize(proto, arena, &size);
+
+  VALUE proto_class = rb_path2class("Google::Protobuf::EnumDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void EnumDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "EnumDescriptor", rb_cObject);
   rb_define_alloc_func(klass, EnumDescriptor_alloc);
@@ -1310,6 +1570,7 @@ static void EnumDescriptor_register(VALUE module) {
   rb_define_method(klass, "file_descriptor", EnumDescriptor_file_descriptor, 0);
   rb_define_method(klass, "is_closed?", EnumDescriptor_is_closed, 0);
   rb_define_method(klass, "options", EnumDescriptor_options, 0);
+  rb_define_method(klass, "to_proto", EnumDescriptor_to_proto, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cEnumDescriptor);
   cEnumDescriptor = klass;
@@ -1356,6 +1617,13 @@ static VALUE ServiceDescriptor_alloc(VALUE klass) {
   return ret;
 }
 
+/**
+ * ruby-doc: ServiceDescriptor
+ *
+ * A ServiceDescriptor provides information about the Protobuf definition of an
+ * RPC service.
+ */
+
 /*
  * call-seq:
  *    ServiceDescriptor.new(c_only_cookie, ptr) => ServiceDescriptor
@@ -1378,10 +1646,11 @@ static VALUE ServiceDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *     ServiceDescriptor.name => name
+ * ruby-doc: ServiceDescriptor#name
  *
  * Returns the name of this service.
+ *
+ * @return [String]
  */
 static VALUE ServiceDescriptor_name(VALUE _self) {
   ServiceDescriptor* self = ruby_to_ServiceDescriptor(_self);
@@ -1389,10 +1658,10 @@ static VALUE ServiceDescriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *    ServiceDescriptor.file_descriptor
+ * ruby-doc: ServiceDescriptor#file_descriptor
  *
- * Returns the FileDescriptor object this service belongs to.
+ * Returns the {FileDescriptor} object this service belongs to.
+ * @return [FileDescriptor]
  */
 static VALUE ServiceDescriptor_file_descriptor(VALUE _self) {
   ServiceDescriptor* self = ruby_to_ServiceDescriptor(_self);
@@ -1401,10 +1670,12 @@ static VALUE ServiceDescriptor_file_descriptor(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     ServiceDescriptor.each(&block)
+ * ruby-doc: ServiceDescriptor#each
  *
  * Iterates over methods in this service, yielding to the block on each one.
+ *
+ * @yield [MethodDescriptor]
+ * @return [nil]
  */
 static VALUE ServiceDescriptor_each(VALUE _self) {
   ServiceDescriptor* self = ruby_to_ServiceDescriptor(_self);
@@ -1419,10 +1690,13 @@ static VALUE ServiceDescriptor_each(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     ServiceDescriptor.options => options
+ * ruby-doc: ServiceDescriptor#options
  *
- * Returns the `ServiceOptions` for this `ServiceDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L901
+ * ServiceOptions} for this {ServiceDescriptor}.
+ *
+ * @return [ServiceOptions]
  */
 static VALUE ServiceDescriptor_options(VALUE _self) {
   ServiceDescriptor* self = ruby_to_ServiceDescriptor(_self);
@@ -1438,6 +1712,30 @@ static VALUE ServiceDescriptor_options(VALUE _self) {
   return service_options;
 }
 
+/*
+ * ruby-doc: ServiceDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L386
+ * ServiceDescriptorProto} of this {ServiceDescriptor}.
+ *
+ * @return [ServiceDescriptorProto]
+ */
+static VALUE ServiceDescriptor_to_proto(VALUE _self) {
+  ServiceDescriptor* self = ruby_to_ServiceDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_ServiceDescriptorProto* proto =
+      upb_ServiceDef_ToProto(self->servicedef, arena);
+  size_t size;
+  const char* serialized =
+      google_protobuf_ServiceDescriptorProto_serialize(proto, arena, &size);
+  VALUE proto_class = rb_path2class("Google::Protobuf::ServiceDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
 static void ServiceDescriptor_register(VALUE module) {
   VALUE klass = rb_define_class_under(module, "ServiceDescriptor", rb_cObject);
   rb_define_alloc_func(klass, ServiceDescriptor_alloc);
@@ -1447,6 +1745,7 @@ static void ServiceDescriptor_register(VALUE module) {
   rb_define_method(klass, "file_descriptor", ServiceDescriptor_file_descriptor,
                    0);
   rb_define_method(klass, "options", ServiceDescriptor_options, 0);
+  rb_define_method(klass, "to_proto", ServiceDescriptor_to_proto, 0);
   rb_include_module(klass, rb_mEnumerable);
   rb_gc_register_address(&cServiceDescriptor);
   cServiceDescriptor = klass;
@@ -1493,6 +1792,13 @@ static VALUE MethodDescriptor_alloc(VALUE klass) {
   return ret;
 }
 
+/**
+ * ruby-doc: MethodDescriptor
+ *
+ * A MethodDescriptor provides information about the Protobuf definition of a
+ * method inside an RPC service.
+ */
+
 /*
  * call-seq:
  *    MethodDescriptor.new(c_only_cookie, ptr) => MethodDescriptor
@@ -1515,10 +1821,11 @@ static VALUE MethodDescriptor_initialize(VALUE _self, VALUE cookie,
 }
 
 /*
- * call-seq:
- *     MethodDescriptor.name => name
+ * ruby-doc: MethodDescriptor#name
  *
  * Returns the name of this method
+ *
+ * @return [String]
  */
 static VALUE MethodDescriptor_name(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1526,10 +1833,13 @@ static VALUE MethodDescriptor_name(VALUE _self) {
 }
 
 /*
- * call-seq:
- *     MethodDescriptor.options => options
+ * ruby-doc: MethodDescriptor#options
  *
- * Returns the `MethodOptions` for this `MethodDescriptor`.
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L927
+ * MethodOptions} for this {MethodDescriptor}.
+ *
+ * @return [MethodOptions]
  */
 static VALUE MethodDescriptor_options(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1546,10 +1856,11 @@ static VALUE MethodDescriptor_options(VALUE _self) {
 }
 
 /*
- * call-seq:
- *      MethodDescriptor.input_type => Descriptor
+ * ruby-doc: MethodDescriptor#input_type
  *
- * Returns the `Descriptor` for the request message type of this method
+ * Returns the {Descriptor} for the request message type of this method
+ *
+ * @return [Descriptor]
  */
 static VALUE MethodDescriptor_input_type(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1558,10 +1869,11 @@ static VALUE MethodDescriptor_input_type(VALUE _self) {
 }
 
 /*
- * call-seq:
- *      MethodDescriptor.output_type => Descriptor
+ * ruby-doc: MethodDescriptor#output_type
  *
- * Returns the `Descriptor` for the response message type of this method
+ * Returns the {Descriptor} for the response message type of this method
+ *
+ * @return [Descriptor]
  */
 static VALUE MethodDescriptor_output_type(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1570,10 +1882,11 @@ static VALUE MethodDescriptor_output_type(VALUE _self) {
 }
 
 /*
- * call-seq:
- *      MethodDescriptor.client_streaming => bool
+ * ruby-doc: MethodDescriptor#client_streaming
  *
  * Returns whether or not this is a streaming request method
+ *
+ * @return [Boolean]
  */
 static VALUE MethodDescriptor_client_streaming(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1581,10 +1894,35 @@ static VALUE MethodDescriptor_client_streaming(VALUE _self) {
 }
 
 /*
- * call-seq:
- *      MethodDescriptor.server_streaming => bool
+ * ruby-doc: MethodDescriptor#to_proto
+ *
+ * Returns the
+ * {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L394
+ * MethodDescriptorProto} of this {MethodDescriptor}.
+ *
+ * @return [MethodDescriptorProto]
+ */
+static VALUE MethodDescriptor_to_proto(VALUE _self) {
+  MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
+  upb_Arena* arena = upb_Arena_New();
+  google_protobuf_MethodDescriptorProto* proto =
+      upb_MethodDef_ToProto(self->methoddef, arena);
+  size_t size;
+  const char* serialized =
+      google_protobuf_MethodDescriptorProto_serialize(proto, arena, &size);
+  VALUE proto_class = rb_path2class("Google::Protobuf::MethodDescriptorProto");
+  VALUE proto_rb =
+      Message_decode_bytes(size, serialized, 0, proto_class, false);
+  upb_Arena_Free(arena);
+  return proto_rb;
+}
+
+/*
+ * ruby-doc: MethodDescriptor#server_streaming
  *
  * Returns whether or not this is a streaming response method
+ *
+ * @return [Boolean]
  */
 static VALUE MethodDescriptor_server_streaming(VALUE _self) {
   MethodDescriptor* self = ruby_to_MethodDescriptor(_self);
@@ -1603,6 +1941,7 @@ static void MethodDescriptor_register(VALUE module) {
                    0);
   rb_define_method(klass, "server_streaming", MethodDescriptor_server_streaming,
                    0);
+  rb_define_method(klass, "to_proto", MethodDescriptor_to_proto, 0);
   rb_gc_register_address(&cMethodDescriptor);
   cMethodDescriptor = klass;
 }
