@@ -12,20 +12,26 @@
 #include "google/protobuf/extension_set.h"
 
 #include <algorithm>
+#include <cctype>
+#include <cstddef>
 #include <cstdint>
+#include <memory>
 #include <string>
+#include <vector>
 
 #include "google/protobuf/descriptor.pb.h"
+#include <gmock/gmock.h>
 #include <gtest/gtest.h>
-#include "absl/base/casts.h"
+#include "absl/algorithm/container.h"
 #include "absl/strings/cord.h"
 #include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
 #include "google/protobuf/arena.h"
 #include "google/protobuf/cpp_features.pb.h"
 #include "google/protobuf/descriptor.h"
 #include "google/protobuf/dynamic_message.h"
 #include "google/protobuf/io/coded_stream.h"
-#include "google/protobuf/io/zero_copy_stream_impl.h"
+#include "google/protobuf/io/zero_copy_stream_impl_lite.h"
 #include "google/protobuf/message_lite.h"
 #include "google/protobuf/port.h"
 #include "google/protobuf/test_util.h"
@@ -51,6 +57,87 @@ namespace internal {
 
 extern bool fully_verify_message_sets_opt_out;
 
+// Enum to represent the type of ExtensionFinder to use in the parameterized
+// test.
+enum class ExtensionFinderType {
+  kGeneratedExtensionFinder,
+  kDescriptorPoolExtensionFinder,
+};
+
+// Parameterized test fixture for FindExtensionInfoFromFieldNumber.
+class FindExtensionTest : public ::testing::TestWithParam<ExtensionFinderType> {
+ protected:
+  std::unique_ptr<GeneratedExtensionFinder> generated_finder_;
+  std::unique_ptr<DescriptorPoolExtensionFinder> pool_finder_;
+  DescriptorPool dynamic_pool_;
+  std::unique_ptr<DynamicMessageFactory> dynamic_factory_;
+  const Message* extendee_;
+
+  FindExtensionTest() : dynamic_pool_(DescriptorPool::generated_pool()) {}
+
+  template <typename MessageT>
+  void ConstructFinders() {
+    extendee_ = &MessageT::default_instance();
+    generated_finder_ = std::make_unique<GeneratedExtensionFinder>(extendee_);
+
+    dynamic_factory_ = std::make_unique<DynamicMessageFactory>(&dynamic_pool_);
+    dynamic_factory_->SetDelegateToGeneratedFactory(true);
+    pool_finder_ = std::make_unique<DescriptorPoolExtensionFinder>(
+        &dynamic_pool_, dynamic_factory_.get(),
+        MessageT::default_instance().GetDescriptor());
+  }
+
+  template <typename T>
+  T* GetFinder() {
+    if constexpr (std::is_same_v<T, GeneratedExtensionFinder>) {
+      return generated_finder_.get();
+    } else if constexpr (std::is_same_v<T, DescriptorPoolExtensionFinder>) {
+      return pool_finder_.get();
+    } else {
+      return nullptr;
+    }
+  }
+
+  bool FindExtensionInfoFromFieldNumber(int wire_type, int field_number,
+                                        ExtensionInfo* extension,
+                                        bool* was_packed_on_wire) {
+    ExtensionSet es;
+    switch (GetParam()) {
+      case ExtensionFinderType::kGeneratedExtensionFinder:
+        return es.FindExtensionInfoFromFieldNumber(
+            wire_type, field_number, GetFinder<GeneratedExtensionFinder>(),
+            extension, was_packed_on_wire);
+      case ExtensionFinderType::kDescriptorPoolExtensionFinder:
+        return es.FindExtensionInfoFromFieldNumber(
+            wire_type, field_number, GetFinder<DescriptorPoolExtensionFinder>(),
+            extension, was_packed_on_wire);
+    }
+    return false;
+  }
+
+  bool CallFindExtension(int wire_type, int field_number,
+                         ExtensionInfo* extension, bool* was_packed_on_wire) {
+    const char* ptr;
+    internal::ParseContext ctx(100, false, &ptr, "");
+    ExtensionSet es;
+    switch (GetParam()) {
+      case ExtensionFinderType::kGeneratedExtensionFinder: {
+        return es.FindExtension(wire_type, field_number, extendee_, &ctx,
+                                extension, was_packed_on_wire);
+      }
+      case ExtensionFinderType::kDescriptorPoolExtensionFinder: {
+        // We need to contrive a ParseContext that will cause the
+        // DescriptorPoolExtensionFinder to be used.
+        ctx.data().pool = &dynamic_pool_;
+        ctx.data().factory = dynamic_factory_.get();
+        return es.FindExtension(wire_type, field_number, extendee_, &ctx,
+                                extension, was_packed_on_wire);
+      }
+    }
+    return false;
+  }
+};
+
 namespace {
 
 using TestUtil::EqualsToSerialized;
@@ -1459,6 +1546,174 @@ TEST(ExtensionSetTest, Descriptor) {
   EXPECT_NE(GetExtensionReflection(pb::cpp), nullptr);
 }
 
+
+TEST_P(FindExtensionTest,
+       FindExtensionInfoFromFieldNumber_FindExistingExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kOptionalInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_TRUE(FindExtensionInfoFromFieldNumber(
+      wire_type, field_number, &extension_info, &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_FALSE(extension_info.is_repeated);
+  EXPECT_FALSE(was_packed_on_wire);
+}
+
+TEST_P(FindExtensionTest, FindExtensionInfoFromFieldNumberExtensionNotFound) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = 9999;  // Non-existent field number
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_FALSE(FindExtensionInfoFromFieldNumber(
+      wire_type, field_number, &extension_info, &was_packed_on_wire));
+}
+
+TEST_P(FindExtensionTest, FindExtensionInfoFromFieldNumberWireTypeMismatch) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kOptionalInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WIRETYPE_FIXED64;  // Incorrect wire type
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_FALSE(FindExtensionInfoFromFieldNumber(
+      wire_type, field_number, &extension_info, &was_packed_on_wire));
+}
+
+TEST_P(FindExtensionTest, FindExtensionInfoFromFieldNumberPackedExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kPackedInt32ExtensionFieldNumber;
+  ConstructFinders<unittest::TestPackedExtensions>();
+
+  // Packed fields appear as LENGTH_DELIMITED on the wire.
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WIRETYPE_LENGTH_DELIMITED;
+
+  EXPECT_TRUE(FindExtensionInfoFromFieldNumber(
+      wire_type, field_number, &extension_info, &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_TRUE(extension_info.is_repeated);
+  EXPECT_TRUE(extension_info.is_packed);
+  EXPECT_TRUE(was_packed_on_wire);
+}
+
+TEST_P(FindExtensionTest,
+       FindExtensionInfoFromFieldNumber_UnpackedRepeatedExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kRepeatedInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_TRUE(FindExtensionInfoFromFieldNumber(
+      wire_type, field_number, &extension_info, &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_TRUE(extension_info.is_repeated);
+  EXPECT_FALSE(extension_info.is_packed);
+  EXPECT_FALSE(was_packed_on_wire);
+}
+
+TEST_P(FindExtensionTest, FindExtensionFindExistingExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kOptionalInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_TRUE(CallFindExtension(wire_type, field_number, &extension_info,
+                                &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_FALSE(extension_info.is_repeated);
+  EXPECT_FALSE(was_packed_on_wire);
+}
+
+TEST_P(FindExtensionTest, FindExtensionExtensionNotFound) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = 9999;  // Non-existent field number
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_FALSE(CallFindExtension(wire_type, field_number, &extension_info,
+                                 &was_packed_on_wire));
+}
+
+TEST_P(FindExtensionTest, FindExtensionWireTypeMismatch) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kOptionalInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WIRETYPE_FIXED64;  // Incorrect wire type
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_FALSE(CallFindExtension(wire_type, field_number, &extension_info,
+                                 &was_packed_on_wire));
+}
+
+TEST_P(FindExtensionTest, FindExtensionPackedExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kPackedInt32ExtensionFieldNumber;
+  ConstructFinders<unittest::TestPackedExtensions>();
+
+  // Packed fields appear as LENGTH_DELIMITED on the wire.
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WIRETYPE_LENGTH_DELIMITED;
+
+  EXPECT_TRUE(CallFindExtension(wire_type, field_number, &extension_info,
+                                &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_TRUE(extension_info.is_repeated);
+  EXPECT_TRUE(extension_info.is_packed);
+  EXPECT_TRUE(was_packed_on_wire);
+}
+
+TEST_P(FindExtensionTest, FindExtensionUnpackedRepeatedExtension) {
+  ExtensionInfo extension_info;
+  bool was_packed_on_wire;
+  const int field_number = unittest::kRepeatedInt32ExtensionFieldNumber;
+  const WireFormatLite::WireType wire_type =
+      WireFormatLite::WireTypeForFieldType(WireFormatLite::TYPE_INT32);
+  ConstructFinders<unittest::TestAllExtensions>();
+
+  EXPECT_TRUE(CallFindExtension(wire_type, field_number, &extension_info,
+                                &was_packed_on_wire));
+  EXPECT_EQ(extension_info.number, field_number);
+  EXPECT_EQ(extension_info.type, WireFormatLite::TYPE_INT32);
+  EXPECT_TRUE(extension_info.is_repeated);
+  EXPECT_FALSE(extension_info.is_packed);
+  EXPECT_FALSE(was_packed_on_wire);
+}
+
+INSTANTIATE_TEST_SUITE_P(
+    ExtensionFinders, FindExtensionTest,
+    ::testing::Values(ExtensionFinderType::kGeneratedExtensionFinder,
+                      ExtensionFinderType::kDescriptorPoolExtensionFinder),
+    [](const testing::TestParamInfo<FindExtensionTest::ParamType>& info) {
+      std::string name = absl::StrCat(
+          info.param == ExtensionFinderType::kGeneratedExtensionFinder
+              ? "GeneratedExtensionFinder"
+              : "DescriptorPoolExtensionFinder");
+      absl::c_replace_if(name, [](char c) { return !std::isalnum(c); }, '_');
+      return name;
+    });
+
 }  // namespace
 }  // namespace internal
 }  // namespace protobuf
